Package: themis
Title: Extra Recipes Steps for Dealing with Unbalanced Data
Version: 1.1.0
Authors@R: c(
    person("Emil", "Hvitfeldt", , "emil.hvitfeldt@posit.co", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-0679-1945")),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: A dataset with an uneven number of cases in each class is
    said to be unbalanced. Many models produce a subpar performance on
    unbalanced datasets. A dataset can be balanced by increasing the
    number of minority cases using SMOTE 2011
    <doi:10.48550/arXiv.1106.1813>, BorderlineSMOTE 2005
    <doi:10.1007/11538059_91> and ADASYN 2008. Or by decreasing the number
    of majority cases using NearMiss 2003
    <https://www.site.uottawa.ca/~nat/Workshop2003/jzhang.pdf> or Tomek
    link removal 1976.
License: MIT + file LICENSE
URL: https://github.com/tidymodels/themis,
        https://themis.tidymodels.org
BugReports: https://github.com/tidymodels/themis/issues
Depends: R (>= 4.1), recipes (>= 1.1.0)
Imports: cli, dplyr, generics (>= 0.1.0), glue, gower, hardhat,
        lifecycle (>= 1.0.3), purrr, RANN, rlang (>= 1.1.0), ROSE,
        tibble, vctrs, withr
Suggests: covr, dials (>= 1.2.0), ggplot2, kernlab, modeldata,
        philentropy, testthat (>= 3.0.0)
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/usethis/last-upkeep: 2025-04-24
Encoding: UTF-8
LazyData: true
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-08-01 23:40:10 UTC; emilhvitfeldt
Author: Emil Hvitfeldt [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-0679-1945>),
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Emil Hvitfeldt <emil.hvitfeldt@posit.co>
Repository: CRAN
Date/Publication: 2026-08-02 05:10:02 UTC
