#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @import rlang
#' @importFrom dplyr all_of
#' @importFrom dplyr bind_rows
#' @importFrom dplyr mutate
#' @importFrom dplyr select
#' @importFrom glue glue
#' @importFrom lifecycle deprecated
#' @importFrom purrr list_rbind
#' @importFrom purrr map_lgl
#' @importFrom recipes add_step
#' @importFrom recipes bake
#' @importFrom recipes check_new_data
#' @importFrom recipes check_type
#' @importFrom recipes is_trained
#' @importFrom recipes prep
#' @importFrom recipes print_step
#' @importFrom recipes rand_id
#' @importFrom recipes recipes_argument_select
#' @importFrom recipes recipes_eval_select
#' @importFrom recipes sel2char
#' @importFrom recipes step
#' @importFrom rlang :=
#' @importFrom rlang caller_arg
#' @importFrom rlang caller_env
#' @importFrom rlang enquos
#' @importFrom ROSE ROSE
#' @importFrom tibble as_tibble
#' @importFrom tibble tibble
#' @importFrom vctrs vec_cbind
#' @importFrom withr with_seed
## usethis namespace: end
NULL
