#' Synthetic Dataset With a Circle
#'
#' A random dataset with two classes one of which is inside a circle. Used for
#' examples to show how the different methods handles borders.
#'
#' @format A data frame with 400 rows and 4 variables:
#' \describe{
#'   \item{x}{Numeric.}
#'   \item{y}{Numeric.}
#'   \item{class}{Factor, values "Circle" and "Rest".}
#'   \item{id}{character, ID variable.}
#' }
#' @source Synthetic data, simulated in \code{data-raw/circle_example.R}.
"circle_example"
