# ncl_impl() errors when too few observations for neighbors

    Code
      ncl_impl(df, var = "class", neighbors = 5)
    Condition
      Error:
      ! Not enough observations to compute 5 nearest neighbors.
      i 3 observations were found, but 6 are needed.

