% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cnn.R
\name{step_cnn}
\alias{step_cnn}
\alias{tidy.step_cnn}
\title{Condensed Nearest Neighbors}
\usage{
step_cnn(
  recipe,
  ...,
  role = NA,
  trained = FALSE,
  column = NULL,
  distance = "euclidean",
  skip = TRUE,
  seed = sample.int(10^5, 1),
  distance_with = recipes::all_predictors(),
  id = rand_id("cnn")
)
}
\arguments{
\item{recipe}{A recipe object. The step will be added to the sequence of
operations for this recipe.}

\item{...}{One or more selector functions to choose which
variable is used to sample the data. See \link[recipes:selections]{recipes::selections}
for more details. The selection should result in \emph{single
factor variable}. For the \code{tidy} method, these are not
currently used.}

\item{role}{Not used by this step since no new variables are
created.}

\item{trained}{A logical to indicate if the quantities for preprocessing have
been estimated.}

\item{column}{A character string of the variable name that will
be populated (eventually) by the \code{...} selectors.}

\item{distance}{A character string specifying the distance metric used for
nearest neighbor calculations, defaulting to \code{"euclidean"}. The available
metrics fall into three groups.

\code{"euclidean"}, \code{"cosine"}, and \code{"mahalanobis"} use approximate nearest
neighbors via the RANN package and scale well to large datasets.

\code{"squared_chord"}, \code{"matusita"}, \code{"hellinger"}, and \code{"bhattacharyya"} are
probability-divergence measures that treat each row as a distribution over
the predictors, so they require non-negative values. \code{"hellinger"} and
\code{"bhattacharyya"} further require each row to sum to 1. All four also use
the RANN package and scale well to large datasets.

\code{"manhattan"}, \code{"chebyshev"}, \code{"canberra"}, \code{"soergel"}, \code{"lorentzian"},
\code{"jeffreys"}, \code{"topsoe"}, \code{"jensen-shannon"}, \code{"jensen_difference"},
\code{"taneja"}, and \code{"kumar-johnson"} compute an exact all-pairs distance
matrix. This takes time and memory proportional to the square of the number
of observations in a class, so these are best suited to smaller datasets.
Everything from \code{"canberra"} onwards is a probability divergence requiring
non-negative values, is provided by the philentropy package (which must be
installed separately), and in the case of \code{"jeffreys"}, \code{"taneja"}, and
\code{"kumar-johnson"} requires strictly positive values, since those divide by
individual predictor values.

The probability divergences are meaningful for compositional predictors such
as proportions or counts normalized per observation, and are generally not
appropriate for standardized predictors.}

\item{skip}{A logical. Should the step be skipped when the recipe is baked by
\code{\link[recipes:bake]{bake()}}? While all operations are baked when \code{\link[recipes:prep]{prep()}} is run, some
operations may not be able to be conducted on new data (e.g. processing the
outcome variable(s)). Care should be taken when using \code{skip = TRUE} as it
may affect the computations for subsequent operations.}

\item{seed}{An integer that will be used as the seed when
applied.}

\item{distance_with}{A call to a selector function to choose
which variables are used for distance calculations. Defaults to
\code{\link[recipes:all_predictors]{recipes::all_predictors()}}. The variable selected by \code{...} is
always excluded from the distance calculations.}

\item{id}{A character string that is unique to this step to identify it.}
}
\value{
An updated version of \code{recipe} with the new step
added to the sequence of existing steps (if any). For the
\code{tidy} method, a tibble with columns \code{terms} which is
the variable used to sample.
}
\description{
\code{step_cnn()} creates a \emph{specification} of a recipe step that removes
redundant majority class observations, keeping only a consistent subset that
correctly classifies the data using a 1-nearest-neighbor rule.
}
\details{
Condensed Nearest Neighbors (CNN) is an under-sampling method that reduces
the majority classes to a consistent subset: a subset that classifies the
original data correctly using a 1-nearest-neighbor rule. It starts with a
"store" containing all minority class observations and one randomly chosen
majority class observation. It then repeatedly scans the remaining majority
class observations and moves any that are misclassified by a 1-nearest
neighbor fit on the current store into the store. This continues until a
full pass adds no new observations. The observations left outside the store
are removed.

The smallest class is treated as the minority class and is always kept. CNN
tends to keep observations near the decision boundary while discarding
redundant interior observations. Because the seed observation and the scan
order are random, results depend on the random seed.

All variables selected by \code{distance_with} must be numeric with no missing
data.

All columns in the data are sampled and returned by \code{\link[recipes:juice]{recipes::juice()}}
and \code{\link[recipes:bake]{recipes::bake()}}.

When used in modeling, users should strongly consider using the
option \code{skip = TRUE} so that the extra sampling is \emph{not}
conducted outside of the training set.
}
\section{Tidying}{
When you \code{\link[recipes:tidy.recipe]{tidy()}} this step, a tibble is returned with
columns \code{terms} and \code{id}:

\describe{
\item{terms}{character, the selectors or variables selected}
\item{id}{character, id of this step}
}
}

\section{Case weights}{


The underlying operation does not allow for case weights. Supplying data
with a case weights column to this step results in an error.
}

\examples{
\dontshow{if (rlang::is_installed("modeldata")) withAutoprint(\{ # examplesIf}
library(recipes)
library(modeldata)
data(hpc_data)

hpc_data0 <- hpc_data |>
  select(-protocol, -day)

orig <- count(hpc_data0, class, name = "orig")
orig

up_rec <- recipe(class ~ ., data = hpc_data0) |>
  step_cnn(class) |>
  prep()

training <- up_rec |>
  bake(new_data = NULL) |>
  count(class, name = "training")
training

# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
  bake(new_data = hpc_data0) |>
  count(class, name = "baked")
baked

orig |>
  left_join(training, by = "class") |>
  left_join(baked, by = "class")

library(ggplot2)

ggplot(circle_example, aes(x, y, color = class)) +
  geom_point() +
  labs(title = "Without CNN") +
  xlim(c(1, 15)) +
  ylim(c(1, 15))

recipe(class ~ x + y, data = circle_example) |>
  step_cnn(class) |>
  prep() |>
  bake(new_data = NULL) |>
  ggplot(aes(x, y, color = class)) +
  geom_point() +
  labs(title = "With CNN") +
  xlim(c(1, 15)) +
  ylim(c(1, 15))
\dontshow{\}) # examplesIf}
}
\references{
Hart, P. (1968). The condensed nearest neighbor rule. IEEE
Transactions on Information Theory, 14(3), 515-516.
}
\seealso{
\code{\link[=cnn]{cnn()}} for direct implementation

Other Steps for under-sampling:
\code{\link[=step_cluster_centroids]{step_cluster_centroids()}},
\code{\link[=step_downsample]{step_downsample()}},
\code{\link[=step_enn]{step_enn()}},
\code{\link[=step_instance_hardness]{step_instance_hardness()}},
\code{\link[=step_ncl]{step_ncl()}},
\code{\link[=step_nearmiss]{step_nearmiss()}},
\code{\link[=step_oss]{step_oss()}},
\code{\link[=step_tomek]{step_tomek()}}
}
\concept{Steps for under-sampling}
