% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adasyn.R, R/bsmote.R, R/cluster_centroids.R,
%   R/downsample.R, R/enn.R, R/instance_hardness.R, R/kmeans_smote.R, R/ncl.R,
%   R/nearmiss.R, R/rose.R, R/smogn.R, R/smote.R, R/smoten.R, R/smotenc.R,
%   R/svmsmote.R, R/tunable.R, R/upsample.R
\name{tunable.step_adasyn}
\alias{tunable.step_adasyn}
\alias{tunable.step_bsmote}
\alias{tunable.step_cluster_centroids}
\alias{tunable.step_downsample}
\alias{tunable.step_enn}
\alias{tunable.step_instance_hardness}
\alias{tunable.step_kmeans_smote}
\alias{tunable.step_ncl}
\alias{tunable.step_nearmiss}
\alias{tunable.step_rose}
\alias{tunable.step_smogn}
\alias{tunable.step_smote}
\alias{tunable.step_smoten}
\alias{tunable.step_smotenc}
\alias{tunable.step_svmsmote}
\alias{tunable_themis}
\alias{tunable.step_upsample}
\title{tunable methods for themis}
\usage{
\method{tunable}{step_adasyn}(x, ...)

\method{tunable}{step_bsmote}(x, ...)

\method{tunable}{step_cluster_centroids}(x, ...)

\method{tunable}{step_downsample}(x, ...)

\method{tunable}{step_enn}(x, ...)

\method{tunable}{step_instance_hardness}(x, ...)

\method{tunable}{step_kmeans_smote}(x, ...)

\method{tunable}{step_ncl}(x, ...)

\method{tunable}{step_nearmiss}(x, ...)

\method{tunable}{step_rose}(x, ...)

\method{tunable}{step_smogn}(x, ...)

\method{tunable}{step_smote}(x, ...)

\method{tunable}{step_smoten}(x, ...)

\method{tunable}{step_smotenc}(x, ...)

\method{tunable}{step_svmsmote}(x, ...)

\method{tunable}{step_upsample}(x, ...)
}
\arguments{
\item{x}{A recipe step object}

\item{...}{Not used.}
}
\value{
A tibble object.
}
\description{
These functions define what parameters \emph{can} be tuned for specific steps.
They also define the recommended objects from the \code{dials} package that can
be used to generate new parameter values and other characteristics.
}
\keyword{internal}
