% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{circle_example}
\alias{circle_example}
\title{Synthetic Dataset With a Circle}
\format{
A data frame with 400 rows and 4 variables:
\describe{
\item{x}{Numeric.}
\item{y}{Numeric.}
\item{class}{Factor, values "Circle" and "Rest".}
\item{id}{character, ID variable.}
}
}
\source{
Synthetic data, simulated in \code{data-raw/circle_example.R}.
}
\usage{
circle_example
}
\description{
A random dataset with two classes one of which is inside a circle. Used for
examples to show how the different methods handles borders.
}
\keyword{datasets}
