% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/upsample.R
\name{step_upsample}
\alias{step_upsample}
\alias{tidy.step_upsample}
\title{Up-Sample a Data Set Based on a Factor Variable}
\usage{
step_upsample(
  recipe,
  ...,
  over_ratio = 1,
  ratio = deprecated(),
  role = NA,
  trained = FALSE,
  column = NULL,
  target = NA,
  indicator_column = NULL,
  skip = TRUE,
  seed = sample.int(10^5, 1),
  id = rand_id("upsample")
)
}
\arguments{
\item{recipe}{A recipe object. The step will be added to the sequence of
operations for this recipe.}

\item{...}{One or more selector functions to choose which
variable is used to sample the data. See \link[recipes:selections]{recipes::selections}
for more details. The selection should result in \emph{single
factor variable}. For the \code{tidy} method, these are not
currently used.}

\item{over_ratio}{A numeric value for the ratio of the
minority-to-majority frequencies. The default value (1) means
that all other levels are sampled up to have the same
frequency as the most occurring level. A value of 0.5 would mean
that the minority levels will have (at most) (approximately)
half as many rows as the majority level.

A named numeric vector can be used instead to give different levels
different targets, for example \code{c(a = 1, b = 0.5)}. The names must be
levels of the outcome and the values are ratios of the majority level,
exactly as in the single-number case. Levels that are not named are left
untouched, as are rows with a missing outcome. Because a vector of targets
is not a single value, supplying one means this argument can no longer be
tuned. See \code{vignette("ratio", package = "themis")} for more details.}

\item{ratio}{Deprecated argument; same as \code{over_ratio}.}

\item{role}{For new variables created by this step, what analysis role
should they be assigned? Only used when \code{indicator_column} is not \code{NULL}.}

\item{trained}{A logical to indicate if the quantities for preprocessing have
been estimated.}

\item{column}{A character string of the variable name that will
be populated (eventually) by the \code{...} selectors.}

\item{target}{A named numeric vector giving the number of rows to sample
each level up to. This should not be set by the user and will be
populated by \code{prep}.}

\item{indicator_column}{A single string or \code{NULL} (the default). If a
string is given, a logical column with that name is added to the output,
marking rows added by the step (\code{TRUE}) vs rows from the original data
(\code{FALSE}).}

\item{skip}{A logical. Should the step be skipped when the recipe is baked by
\code{\link[recipes:bake]{bake()}}? While all operations are baked when \code{\link[recipes:prep]{prep()}} is run, some
operations may not be able to be conducted on new data (e.g. processing the
outcome variable(s)). Care should be taken when using \code{skip = TRUE} as it
may affect the computations for subsequent operations.}

\item{seed}{An integer that will be used as the seed when applied.}

\item{id}{A character string that is unique to this step to identify it.}
}
\value{
An updated version of \code{recipe} with the new step
added to the sequence of existing steps (if any). For the
\code{tidy} method, a tibble with columns \code{terms} which is
the variable used to sample.
}
\description{
\code{step_upsample()} creates a \emph{specification} of a recipe step that will
replicate rows of a data set to make the occurrence of levels in a specific
factor level equal.
}
\details{
Up-sampling is intended to be performed on the \emph{training} set
alone. For this reason, the default is \code{skip = TRUE}.

If there are missing values in the factor variable that is used
to define the sampling, missing data are selected at random in
the same way that the other factor levels are sampled. Missing
values are not used to determine the amount of data in the
majority level (see example below).

For any data with factor levels occurring with the same
frequency as the majority level, all data will be retained.

All columns in the data are sampled and returned by \code{\link[recipes:juice]{recipes::juice()}}
and \code{\link[recipes:bake]{recipes::bake()}}.
}
\section{Tidying}{
When you \code{\link[recipes:tidy.recipe]{tidy()}} this step, a tibble is returned with
columns \code{terms} and \code{id}:

\describe{
\item{terms}{character, the selectors or variables selected}
\item{id}{character, id of this step}
}
}

\section{Tuning Parameters}{
This step has 1 tuning parameters:
\itemize{
\item \code{over_ratio}: Over-Sampling Ratio (type: double, default: 1)
}
}

\section{Case weights}{


This step performs an unsupervised operation that can utilize case weights.
To use them, see the documentation in \link[recipes:case_weights]{recipes::case_weights} and the examples on
\code{tidymodels.org}.
}

\examples{
\dontshow{if (rlang::is_installed("modeldata")) withAutoprint(\{ # examplesIf}
library(recipes)
library(modeldata)
data(hpc_data)

hpc_data0 <- hpc_data |>
  select(-protocol, -day)

orig <- count(hpc_data0, class, name = "orig")
orig

up_rec <- recipe(class ~ ., data = hpc_data0) |>
  # Bring the minority levels up to about 1000 each
  # 1000/2211 is approx 0.4523
  step_upsample(class, over_ratio = 0.4523) |>
  prep()

training <- up_rec |>
  bake(new_data = NULL) |>
  count(class, name = "training")
training

# Since `skip` defaults to TRUE, baking the step has no effect
baked <- up_rec |>
  bake(new_data = hpc_data0) |>
  count(class, name = "baked")
baked

# Note that if the original data contained more rows than the
# target n (= ratio * majority_n), the data are left alone:
orig |>
  left_join(training, by = "class") |>
  left_join(baked, by = "class")

library(ggplot2)

ggplot(circle_example, aes(x, y, color = class)) +
  geom_point() +
  labs(title = "Without upsample")

recipe(class ~ x + y, data = circle_example) |>
  step_upsample(class) |>
  prep() |>
  bake(new_data = NULL) |>
  ggplot(aes(x, y, color = class)) +
  geom_jitter(width = 0.1, height = 0.1) +
  labs(title = "With upsample (with jittering)")
\dontshow{\}) # examplesIf}
}
\seealso{
Other Steps for over-sampling:
\code{\link[=step_adasyn]{step_adasyn()}},
\code{\link[=step_bsmote]{step_bsmote()}},
\code{\link[=step_kmeans_smote]{step_kmeans_smote()}},
\code{\link[=step_rose]{step_rose()}},
\code{\link[=step_smogn]{step_smogn()}},
\code{\link[=step_smote]{step_smote()}},
\code{\link[=step_smoten]{step_smoten()}},
\code{\link[=step_smotenc]{step_smotenc()}},
\code{\link[=step_svmsmote]{step_svmsmote()}}
}
\concept{Steps for over-sampling}
