% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adasyn.R, R/bsmote.R, R/cluster_centroids.R,
%   R/cnn.R, R/downsample.R, R/enn.R, R/instance_hardness.R, R/kmeans_smote.R,
%   R/ncl.R, R/nearmiss.R, R/oss.R, R/rose.R, R/smogn.R, R/smote.R, R/smoten.R,
%   R/smotenc.R, R/svmsmote.R, R/tomek.R, R/upsample.R
\name{required_pkgs.step_adasyn}
\alias{required_pkgs.step_adasyn}
\alias{required_pkgs.step_bsmote}
\alias{required_pkgs.step_cluster_centroids}
\alias{required_pkgs.step_cnn}
\alias{required_pkgs.step_downsample}
\alias{required_pkgs.step_enn}
\alias{required_pkgs.step_instance_hardness}
\alias{required_pkgs.step_kmeans_smote}
\alias{required_pkgs.step_ncl}
\alias{required_pkgs.step_nearmiss}
\alias{required_pkgs.step_oss}
\alias{required_pkgs.step_rose}
\alias{required_pkgs.step_smogn}
\alias{required_pkgs.step_smote}
\alias{required_pkgs.step_smoten}
\alias{required_pkgs.step_smotenc}
\alias{required_pkgs.step_svmsmote}
\alias{required_pkgs.step_tomek}
\alias{required_pkgs.step_upsample}
\title{S3 methods for tracking which additional packages are needed for steps.}
\usage{
\method{required_pkgs}{step_adasyn}(x, ...)

\method{required_pkgs}{step_bsmote}(x, ...)

\method{required_pkgs}{step_cluster_centroids}(x, ...)

\method{required_pkgs}{step_cnn}(x, ...)

\method{required_pkgs}{step_downsample}(x, ...)

\method{required_pkgs}{step_enn}(x, ...)

\method{required_pkgs}{step_instance_hardness}(x, ...)

\method{required_pkgs}{step_kmeans_smote}(x, ...)

\method{required_pkgs}{step_ncl}(x, ...)

\method{required_pkgs}{step_nearmiss}(x, ...)

\method{required_pkgs}{step_oss}(x, ...)

\method{required_pkgs}{step_rose}(x, ...)

\method{required_pkgs}{step_smogn}(x, ...)

\method{required_pkgs}{step_smote}(x, ...)

\method{required_pkgs}{step_smoten}(x, ...)

\method{required_pkgs}{step_smotenc}(x, ...)

\method{required_pkgs}{step_svmsmote}(x, ...)

\method{required_pkgs}{step_tomek}(x, ...)

\method{required_pkgs}{step_upsample}(x, ...)
}
\arguments{
\item{x}{A recipe step}
}
\value{
A character vector
}
\description{
S3 methods for tracking which additional packages are needed for steps.
}
\keyword{internal}
